//  Diffusion Client Library for iOS, OS X and tvOS
//
//  Copyright (c) 2015, 2016 Push Technology Ltd., All Rights Reserved.
//
//  Use is subject to license terms.
//
//  NOTICE: All information contained herein is, and remains the
//  property of Push Technology. The intellectual and technical
//  concepts contained herein are proprietary to Push Technology and
//  may be covered by U.S. and Foreign Patents, patents in process, and
//  are protected by trade secret or copyright law.

@import Foundation;

/**
 The type of a topic.

 @since 5.6
 */
typedef NS_ENUM(NSUInteger, PTDiffusionTopicType) {
    /**
     A stateless topic is one that has no data state maintained at the server
     and performs no specific function. Such a topic would normally be used
     simply to act an organisational node within the topic tree but may also
     be used for sending messages.

     @since 5.6
     */
    PTDiffusionTopicType_Stateless = 1,

    /**
     This is a stateful topic that maintains its state as a
     single String value which may optionally be validated and constrained by
     type (e.g. integer, decimal or custom values).

     @since 5.6
     */
    PTDiffusionTopicType_SingleValue = 2,

    /**
     A functional topic that automatically maintains a list of its children and
     notifies subscribers of changes when a child is added or removed.

     Only users of the classic API can access such topics.

     @deprecated Since 5.9. Will be removed in a future release.

     @since 5.6
     */
    PTDiffusionTopicType_ChildList __deprecated_enum_msg("Will be removed in a future release.") = 3,

    /**
     * This is a stateful topic that maintains its state
     * in record format. The format of the topic content may be one or
     * more Records as defined by an item of metadata. When
     * such a topic is updated then the metadata is used to interpret the data
     * and make comparisons between the current topic state and the update such
     * that a delta of change can be calculated for publishing to subscribed
     * clients.

     @since 5.6
     */
    PTDiffusionTopicType_Record = 4,

    /**
     A stateful topic that maintains its state in Google Protocol Buffers format.
     The format of the data is described by a single Protocol Buffers message
     description. When such a topic is updated, the message description is used to
     interpret the data and make comparisons between the current topic state and the
     update such that a delta of change can be calculated for publishing to
     subscribed clients.

     @deprecated Since 5.9. Support for Google Protocol Buffer topics will be
     removed in a future release as Binary topics can be used to distribute
     protocol buffer messages.

     @since 5.7
     */
    PTDiffusionTopicType_ProtocolBuffer __deprecated_enum_msg("Will be removed in a future release.") = 5,

    /**
     A stateful topic that maintains its state via a user written Java class
     deployed at the server.

     @deprecated Since 5.9. Will be removed in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_Custom __deprecated_enum_msg("Will be removed in a future release.") = 6,

    /**
     A stateful topic that links to another topic which has data (i.e. an alias).

     Though such a topic maintains no state itself it is seen as stateful as it is
     permissible to update it (which is effectively the same as updating target topic).
     */
    PTDiffusionTopicType_Slave = 7,

    /**
     A functional topic that enables asynchronous request/response calls.

     Such a topic will have its functionality provided by a user written Java
     class deployed at the server.
     
     @note The capability to retrieve topic details for service topics is provided
     for legacy purposes only as they have been deprecated since 5.7. They cannot
     be created using this client library.

     @deprecated Since 5.9.2. Will be removed in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_Service __deprecated_enum_msg("Will be removed in a future release.") = 8,

    /**
     A functional topic that maintains its state as a list of Strings which
     individual clients can page through and be dynamically notified of changes.

     Paged topics may only be used by users of the classic API.

     Such a topic may be updated by a client using the paging control feature,
     which is not available in this client library.

     @deprecated Since 5.9. Will be removed in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_PagedString __deprecated_enum_msg("Will be removed in a future release.") = 9,

    /**
     A functional Topic that maintains its state as a List of Records which
     individual Clients can page through and be dynamically notified of changes.

     Paged topics may only be used by users of the classic API.

     Such a topic may be updated by a client using the paging control feature,
     which is not available in this client library.

     @deprecated Since 5.9. Will be removed in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_PagedRecord __deprecated_enum_msg("Will be removed in a future release.") = 10,

    /**
     A functional topic that allows clients to be notified of the addition and
     removal of other topics.

     Only users of the classic API can access such topics.
     
     @deprecated Since 5.9. Will be removed in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_TopicNotify __deprecated_enum_msg("Will be removed in a future release.") = 11,

    /**
     A functional topic that can point to different target topics for different clients.

     From the point of view of a client subscribing to such a topic this would
     be seen as a normal stateful topic but it has no state of its own and cannot be
     published to.

     Such a topic may specify a user written Java class which will be invoked to define
     the mapping of the topic to another data topic when a client subscribes.
     Alternatively the mapping can be delegated to a control client using the
     subscription control feature, which is not available in this client library.

     @since 5.7
     */
    PTDiffusionTopicType_Routing = 12,

    /**
     Binary Topic.

     This is a stateful topic that handles data in Binary format.

     @note It is only possible to subscribe and receive opaque content for JSON topics
     with this release of this client library. Full support for this topic type
     will come in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_Binary = 13,

    /**
     JSON (JavaScript Object Notation) Topic.

     This is a stateful topic that handles data in JSON representation.
     
     @note It is only possible to subscribe and receive opaque content for JSON topics
     with this release of this client library. Full support for this topic type
     will come in a future release.

     @since 5.7
     */
    PTDiffusionTopicType_JSON = 14,
};

NS_ASSUME_NONNULL_BEGIN

/**
 Indicates whether the given topic type is stateful.

 A stateful topic has data managed at the server.
 
 @param topicType The topic type for which statefulness is to be queried.

 @since 5.7
 */
BOOL PTDiffusionTopicTypeIsStateful(PTDiffusionTopicType topicType);

/**
 Indicates whether the given topic type is functional.

 Generally speaking a functional topic is one that performs some function
 other than maintaining and publishing data in the normal manner. Some
 functional topics have no state at the server and simply enable certain
 features whereas others may maintain some server side state but this
 state may only be updated using specialised features.
 
 @param topicType The topic type for which functionalness is to be queried.

 @since 5.7
 */
BOOL PTDiffusionTopicTypeIsFunctional(PTDiffusionTopicType topicType);

/**
 Returns a string representation of the given topic type.

 @param topicType The topic type for which a description is to be returned.

 @since 5.7
 */
NSString* PTDiffusionTopicTypeToString(PTDiffusionTopicType topicType);

/**
 Returns the name of this topic type constant, exactly as declared in its
 enum declaration in the Diffusion API for Java.

 @param topicType The topic type for which the name is to be returned.

 @since 5.9
 */
NSString* PTDiffusionTopicTypeToName(PTDiffusionTopicType topicType);

NS_ASSUME_NONNULL_END
